/*******************************************************************
*
*  DESCRIPTION: class Motor 
*
*  AUTHOR: Herrero Gustavo
*
*  EMAIL: mailto://gh4o@dc.uba.ar
*
*  DATE: 19/04/2002
*
*******************************************************************/

/** include files **/
#include "deftp1.h"
#include "motor.h"     // base header
#include "message.h"   // class InternalMessage 
#include "parsimu.h"   // class Simulator
//#include "strutil.h"   // str2Value( ... )

//#define m_detenido  0
//#define m_subiendo  1
//#define m_bajando  -1
//#define m_parando   2


/*******************************************************************
* CLASS MotorState
*********************************************************************/
/*******************************************************************
* Function Name: operator =
*********************************************************************/
MotorState& MotorState::operator=(MotorState& thisState) {

	(AtomicState &)*this	= (AtomicState &) thisState;
	this->s_motor			= thisState.s_motor;
	this->cantidad_pisos	= thisState.cantidad_pisos;
	return *this;
}

/*******************************************************************
* Function Name: copyState
*********************************************************************/
void MotorState::copyState(MotorState *rhs) {
	*this = *((MotorState *) rhs);	
}

/*******************************************************************
* Function Name: getSize
*********************************************************************/
int  MotorState::getSize() const {
	return sizeof( MotorState );
}


/*******************************************************************
* CLASS Motor
*********************************************************************/

/*******************************************************************
* Function Name: Motor
* Description: constructor
********************************************************************/

Motor::Motor( const string &name )
	: Atomic( name )
	, subir( addInputPort( "subir" ) )
	, bajar( addInputPort( "bajar" ) )
	, respuesta( addOutputPort( "respuesta" ) )
{

	tiempo_avance = "0:0:8:0" ;
	tiempo_frenado = "0:0:2:0" ;

	if( ParallelMainSimulator::Instance().existsParameter( description(), "tiempo_avance" ) )
		tiempo_avance = ParallelMainSimulator::Instance().getParameter( description(), "tiempo_avance" ) ;

	if( ParallelMainSimulator::Instance().existsParameter( description(), "tiempo_frenado" ) )
		tiempo_frenado = ParallelMainSimulator::Instance().getParameter( description(), "tiempo_frenado" ) ;

}


/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &Motor::initFunction()
{

	s_motor(m_detenido);
	cantidad_pisos(0);	
	this->passivate();
	return *this ;
}


/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &Motor::externalFunction( const ExternalMessage &msg )
{

	if( msg.port() == subir && msg.value() > 0)
	{
		if( s_motor() == m_detenido ) {
			// si el motor esta detenido ponerlo en marcha y subir la cabina.
			s_motor(m_subiendo);
			cantidad_pisos((int) msg.value());
			holdIn(AtomicState::active, t_avance());
			
		} else{
			if( s_motor() == m_subiendo ){
				// se sube de a un piso por vez,
				// entonces veo si el pedido es anterior al proximo piso a parar.
				if (cantidad_pisos() > msg.value()) 
					//indicar que se para en un piso anterior.
					cantidad_pisos((int) msg.value());
				}
			//No hace falta programar la proxima transicion interna, porque se avanza de a 1 piso
			//calcular cuanto tiempo le falta para pasar el proximo piso
			//holdIn( AtomicState::active, nextChange() - msg.time());
		}
	}
	if( msg.port() == bajar && msg.value() > 0) {
		if( s_motor() == m_detenido ) {
			// si el motor esta detenido ponerlo en marcha y bajar la cabina.
			s_motor(m_bajando);
			cantidad_pisos((int) msg.value());
			holdIn(AtomicState::active, t_avance());
		} else {
			if( s_motor() == m_bajando ){
				// se sube de a un piso por vez,
				// entonces veo si el pedido es anterior al proximo piso a parar.
				if (cantidad_pisos() > msg.value())
					//indicar que se para en unpiso anterior.
					cantidad_pisos((int) msg.value());
			}
			//No hace falta programar la proxima transicion interna, porque se avanza de a 1 piso
			//calcular cuanto tiempo le falta para pasar el proximo piso
			//holdIn(state(), nextChange() - msg.time());
		}
	}

	return *this;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &Motor::internalFunction( const InternalMessage & )
{
	if( s_motor() != m_detenido ) {//si no esta detenido
		if (s_motor() == m_parando) {
		s_motor(m_detenido);
		this->passivate();
		}
		else {
			cantidad_pisos( cantidad_pisos() - 1); 
			//si llego al destino, frenar para detenerse
			if (cantidad_pisos() == 0) { 
				s_motor(m_parando);
				holdIn( AtomicState::active, t_frenado());
			} 
			else //seguir avanzando un piso ms.
			holdIn( AtomicState::active, t_avance());
		}
	}

	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &Motor::outputFunction( const CollectMessage &msg )
{
	
	int valor_salida;

	if (s_motor() == m_parando)
		// s_motor() es  0 detenido. 
		valor_salida = m_detenido;
	else 
		//  s_motor() es 1 si est subiendo, -1 si baja
		valor_salida = s_motor();

	sendOutput( msg.time(), respuesta,  valor_salida) ;

	return *this ;
}
